#!/bin/bash

# Slackware build script for qemu

# Copyright 2009, 2010  Andrew Brouwers <abrouwers@gmail.com>
# Copyright 2013-2022  Edward W. Koenig, Vancouver, USA <kingbeowulf@linuxgalaxy.org>
# All rights reserved.
#
# Redistribution and use of this script, with or without modification, is
# permitted provided that the following conditions are met:
#
# 1. Redistributions of this script must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ''AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# 20230701 bkw: Modified by SlackBuilds.org, BUILD=2:
# - add doinst to update desktop database and icon cache.
# Note: sbopkglint <= 0.9.0 complains about ELF binaries found in
# /usr/share/qemu (intended for the target systems). This is an error
# in sbopkglint, and will be fixed (no need to change anything here).

cd $(dirname $0) ; CWD=$(pwd)

# Included in Slint by Didier Spaier didieratslintdotfr

# Last build with these installed runtime deps:
# capstone
# libblkio
# libcacard
# libiscsi
# libslirp
# pcsc-lite (dep of libcacard)
# virglrenderer
# depfinder finds them all

PRGNAM=qemu
VERSION=${VERSION:-8.2.2}
BUILD=1
TAG=slint
PKGTYPE=${PKGTYPE:-txz}

KVMGROUP=${KVMGROUP:-users}
TARGETS=$(cat targets)
AUDIODRIVERS=${AUDIODRIVERS:-pa,alsa,oss,sdl}
BRIDGE_HELPER_SETUID=yes

if [ -z "$ARCH" ]; then
  case "$( uname -m )" in
    i?86) ARCH=i586 ;;
    arm*) ARCH=arm ;;
       *) ARCH=$( uname -m ) ;;
  esac
fi

if [ ! -z "${PRINT_PACKAGE_NAME}" ]; then
  echo "$PRGNAM-$VERSION-$ARCH-$BUILD$TAG.$PKGTYPE"
  exit 0
fi

TMP=$CWD
PKG=$TMP/package-$PRGNAM
OUTPUT=$CWD

if [ "$ARCH" = "i586" ]; then
  SLKCFLAGS="-O2 -march=i586 -mtune=i686"
  LIBDIRSUFFIX=""
  KVMARCH="i386"
elif [ "$ARCH" = "i686" ]; then
  SLKCFLAGS="-O2 -march=i686 -mtune=i686"
  LIBDIRSUFFIX=""
  KVMARCH="i386"
elif [ "$ARCH" = "x86_64" ]; then
  SLKCFLAGS="-O2 -fPIC"
  LIBDIRSUFFIX="64"
  KVMARCH="x86_64"
else
  SLKCFLAGS="-O2"
  LIBDIRSUFFIX=""
fi

# Needed to build man pages if built after plain "su"
export PATH=$PATH:/usr/share/texmf/bin

# Autodetect liburing support
# uring="dis" ; if pkg-config --exists liburing ; then uring="en" ; fi

set -e

rm -rf $PKG
mkdir -p $TMP $PKG $OUTPUT
cd $TMP
rm -rf $PRGNAM-$VERSION
tar xf $CWD/$PRGNAM-$VERSION.tar.?z?
cd $PRGNAM-$VERSION
chown -R root:root .
find -L . \
 \( -perm 777 -o -perm 775 -o -perm 750 -o -perm 711 -o -perm 555 \
  -o -perm 511 \) -exec chmod 755 {} + -o \
 \( -perm 666 -o -perm 664 -o -perm 640 -o -perm 600 -o -perm 444 \
  -o -perm 440 -o -perm 400 \) -exec chmod 644 {} + || true

# Remove double CFLAGS
sed -i "s|^\ \ CFLAGS=\"-O2\ |  CFLAGS=\"|" configure

# Enable only x86* and arm emulation for qemu: if you want to build
# all the targets available use TARGETS=all
# if [ "$TARGETS" = "all" ]; then
#  targets=""
# else
#  targets="--target-list=$TARGETS"
# fi

# To omit VNC capability, use VNC_ENABLE=no
#if [ "$VNC_ENABLE" = "yes" ]; then
#  with_vnc="--enable-vnc --enable-vnc-sasl --enable-vnc-jpeg"
#else
#  with_vnc="--disable-vnc"
#fi
#
## To enable user mode networking use SLIRP=yes
#if [ "$SLIRP" = "yes" ]; then
#  with_slirp="--enable-slirp"
#else
#  with_slirp=""
#fi

CFLAGS="$SLKCFLAGS" \
CXXFLAGS="$SLKCFLAGS" \
./configure \
  --prefix=/usr \
  --libdir=/usr/lib${LIBDIRSUFFIX} \
  --sysconfdir=/etc \
  --localstatedir=/var \
  --docdir=/usr/doc/$PRGNAM-$VERSION \
  --audio-drv-list=${AUDIODRIVERS} \
  --enable-malloc=jemalloc \
  --enable-nettle \
  --target-list=$TARGETS

make V=1 -j9
make install DESTDIR=$PKG

# Remove empty /var
rm -rf $PKG/var

# install some udev rules for /dev/kvm
# Note: no udev rules for usb pasthrough when running qemu as non-root on linux hosts. See Section 2.11.2
# https://qemu.weilnetz.de/doc/qemu-doc.html
mkdir -p $PKG/lib/udev/rules.d/
sed "s/@GROUP@/$KVMGROUP/" $CWD/65-kvm.rules \
  > $PKG/lib/udev/rules.d/65-kvm.rules

find $PKG -print0 | xargs -0 file | grep -e "executable" -e "shared object" | grep ELF \
  | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null || true

# create a symlink for qemu-kvm to support aqemu and such frontends
# even though qemu now has a gtk gui.
[ ! -z $KVMARCH ] && ln -s qemu-system-$KVMARCH $PKG/usr/bin/qemu-kvm

# set qemu-bridge-helper setuid if requested
if [ "$BRIDGE_HELPER_SETUID" = "yes" ]; then
  chmod u+s $PKG/usr/libexec/qemu-bridge-helper
fi

# move any generated man pages to their proper location
if [ -d $PKG/usr/share/man ]; then
  mv $PKG/usr/share/man $PKG/usr
  gzip -9 $PKG/usr/man/man?/*.?
fi

# Add docs to the proper location
mkdir -p $PKG/usr/doc/$PRGNAM-$VERSION
cp -a COPYING* LICENSE MAINTAINERS README.rst docs/* \
  $PKG/usr/doc/$PRGNAM-$VERSION
cat $CWD/$PRGNAM.SlackBuild > $PKG/usr/doc/$PRGNAM-$VERSION/$PRGNAM.SlackBuild

mkdir -p $PKG/install
cat $CWD/slack-desc > $PKG/install/slack-desc
cat $CWD/doinst.sh > $PKG/install/doinst.sh

cd $PKG
/sbin/makepkg -l y -c n $OUTPUT/$PRGNAM-$VERSION-$ARCH-$BUILD$TAG.$PKGTYPE
